O_FILE_PATH = ""
IN_FILE_PATH = ""

O_FILE_FOLDER_NAME = 'o_files'
IN_FILE_FOLDER_NAME = 'in_files'
GRAPH_FILES_FOLDER_NAME = 'graph_files'

EDGES_FILE_PREFIX = "edges"
NODES_FILE_PREFIX = "nodes"

ID_NO_MAPPING_FILE_NAME = 'ids_no_mapping.tsv'
STATS_FILE_NAME = 'stats.txt'
TN_ID_NO_MAPPING_FILE_NAME = 'tn_ids_no_mapping.tsv'
TN_STATS_FILE_NAME = 'tn_stats.txt'
