from openbiolink.graph_creation.metadata_edge.onto.edgeMetaAnatomyOntoIsA import EdgeMetaAnatomyOntoIsA
from openbiolink.graph_creation.metadata_edge.onto.edgeMetaAnatomyOntoPartOf import EdgeMetaAnatomyOntoPartOf
from openbiolink.graph_creation.metadata_edge.onto.edgeMetaDisOntoIsA import EdgeMetaDisOntoIsA
from openbiolink.graph_creation.metadata_edge.onto.edgeMetaGoOntoIsA import EdgeMetaGoOntoIsA
from openbiolink.graph_creation.metadata_edge.onto.edgeMetaGoOntoPartOf import EdgeMetaGoOntoPartOf
from openbiolink.graph_creation.metadata_edge.onto.edgeMetaPhenoOntoIsA import EdgeMetaPhenoOntoIsA
