from openbiolink.graph_creation.file_reader.edge.edgeBgeeDiffReader import EdgeBgeeDiffReader
from openbiolink.graph_creation.file_reader.edge.edgeBgeeReader import EdgeBgeeReader
from openbiolink.graph_creation.file_reader.edge.edgeCdtPathReader import EdgeCdtPathReader
from openbiolink.graph_creation.file_reader.edge.edgeDisGeNetReader import EdgeDisGeNetReader
from openbiolink.graph_creation.file_reader.edge.edgeDrugCentralReader import EdgeDrugCentralReader
from openbiolink.graph_creation.file_reader.edge.edgeGoReader import EdgeGoReader
# from openbiolink.graph_creation.file_reader.edge.edgeHpaReader import       EdgeHpaReader --> replaced by bgee
from openbiolink.graph_creation.file_reader.edge.edgeHpoDisReader import EdgeHpoDisReader
# from openbiolink.graph_creation.file_reader.edge.edgeSiderIndReader import  EdgeSiderIndReader --> replaced by drugcentral
from openbiolink.graph_creation.file_reader.edge.edgeHpoGeneReader import EdgeHpoGeneReader
from openbiolink.graph_creation.file_reader.edge.edgeSiderSeReader import EdgeSiderSeReader
from openbiolink.graph_creation.file_reader.edge.edgeStitchActionReader import EdgeStitchActionReader
from openbiolink.graph_creation.file_reader.edge.edgeStitchReader import EdgeStitchReader
from openbiolink.graph_creation.file_reader.edge.edgeStringActionReader import EdgeStringActionReader
from openbiolink.graph_creation.file_reader.edge.edgeStringReader import EdgeStringReader
from openbiolink.graph_creation.file_reader.edge.edgeTnHpoDisReader import EdgeTnHpoDisReader
