from openbiolink.graph_creation.metadata_db_file.onto.dbMetaOntoDo import DbMetaOntoDo
from openbiolink.graph_creation.metadata_db_file.onto.dbMetaOntoGo import DbMetaOntoGo
from openbiolink.graph_creation.metadata_db_file.onto.dbMetaOntoHpo import DbMetaOntoHpo
from openbiolink.graph_creation.metadata_db_file.onto.dbMetaOntoUberon import DbMetaOntoUberon
from openbiolink.graph_creation.metadata_db_file.onto.dbMetadataOnto import DbMetadataOnto
