from openbiolink.graph_creation.metadata_infile.onto.inMetaOntoDoIsA import InMetaOntoDoIsA
from openbiolink.graph_creation.metadata_infile.onto.inMetaOntoGoIsA import InMetaOntoGoIsA
from openbiolink.graph_creation.metadata_infile.onto.inMetaOntoGoPartOf import InMetaOntoGoPartOf
from openbiolink.graph_creation.metadata_infile.onto.inMetaOntoHpoIsA import InMetaOntoHpoIsA
from openbiolink.graph_creation.metadata_infile.onto.inMetaOntoUberonIsA import InMetaOntoUberonIsA
from openbiolink.graph_creation.metadata_infile.onto.inMetaOntoUberonPartOf import InMetaOntoUberonPartOf
