from openbiolink.graph_creation.file_processor.onto.ontoDoIsAProcessor import OntoDoIsAProcessor
from openbiolink.graph_creation.file_processor.onto.ontoGoIsAProcessor import OntoGoIsAProcessor
from openbiolink.graph_creation.file_processor.onto.ontoGoPartOfProcessor import OntoGoPartOfProcessor
from openbiolink.graph_creation.file_processor.onto.ontoHpoIsAProcessor import OntoHpoIsAProcessor
from openbiolink.graph_creation.file_processor.onto.ontoUberonIsAProcessor import OntoUberonIsAProcessor
from openbiolink.graph_creation.file_processor.onto.ontoUberonPartOfProcessor import OntoUberonPartOfProcessor
