from openbiolink.graph_creation.file_processor.mapping.mapDisGeNetProcessor import MapDisGeNetProcessor
from openbiolink.graph_creation.file_processor.mapping.mapDrugCentralPubchemProcessor import MapDrugCentralPubchemProcessor
from openbiolink.graph_creation.file_processor.mapping.mapStringProcessor import MapStringProcessor
from openbiolink.graph_creation.file_processor.mapping.mapUniEnsNcbiProcessor import MapUniEnsNcbiProcessor
from openbiolink.graph_creation.file_processor.mapping.mapUniUniNcbiProcessor import MapUniUniNcbiProcessor
