from openbiolink.graph_creation.file_reader.mapping.mapDisGeNetReader import MapDisGeNetReader
from openbiolink.graph_creation.file_reader.mapping.mapDrugCentralPubchemReader import MapDrugCentralPubchemReader
from openbiolink.graph_creation.file_reader.mapping.mapStringReader import MapStringReader
from openbiolink.graph_creation.file_reader.mapping.mapUniprotReader import MapUniprotReader
