WORKING_DIR = ''
LOG_FILE_PATH = ''
GUI_MODE = False
RANDOM_STATE = 42  # do not change for reproducibility
GRAPH_PROP_FILE_NAME = 'graph_properties.json'

INTERACTIVE_MODE = True
SKIP_EXISTING_FILES = True

# graph_files col names
NODE1_ID_COL_NAME = 'id1'
NODE2_ID_COL_NAME = 'id2'
EDGE_TYPE_COL_NAME = 'edgeType'
QSCORE_COL_NAME = 'qscore'
VALUE_COL_NAME = 'value'
SCORE_COL_NAME = 'score'
COL_NAMES_TRIPLES = [NODE1_ID_COL_NAME, EDGE_TYPE_COL_NAME, NODE2_ID_COL_NAME]
COL_NAMES_EDGES = [NODE1_ID_COL_NAME, EDGE_TYPE_COL_NAME, NODE2_ID_COL_NAME, QSCORE_COL_NAME]
COL_NAMES_SAMPLES = [NODE1_ID_COL_NAME, EDGE_TYPE_COL_NAME, NODE2_ID_COL_NAME, QSCORE_COL_NAME, VALUE_COL_NAME]

# node files col names
ID_NODE_COL_NAME = 'id'
NODE_TYPE_COL_NAME = 'nodeType'
COL_NAMES_NODES = [ID_NODE_COL_NAME, NODE_TYPE_COL_NAME]
