from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDisDrug import EdgeMetaDisDrug
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDisPheno import EdgeMetaDisPheno
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugActivationGene import EdgeMetaDrugActivationGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugBindActGene import EdgeMetaDrugBindActGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugBindInhGene import EdgeMetaDrugBindInhGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugBindingGene import EdgeMetaDrugBindingGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugCatalysisGene import EdgeMetaDrugCatalysisGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugExpressionGene import EdgeMetaDrugExpressionGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugInhibitionGene import EdgeMetaDrugInhibitionGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugPheno import EdgeMetaDrugPheno
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugPredBindGene import EdgeMetaDrugPredBindGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaDrugReactionGene import EdgeMetaDrugReactionGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneActivationGene import EdgeMetaGeneActivationGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneAna import EdgeMetaGeneAna
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneBindActGene import EdgeMetaGeneBindActGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneBindInhGene import EdgeMetaGeneBindInhGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneBindingGene import EdgeMetaGeneBindingGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneCatalysisGene import EdgeMetaGeneCatalysisGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneDis import EdgeMetaGeneDis
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneDrug import EdgeMetaGeneDrug
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneExpressionGene import EdgeMetaGeneExpressionGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneGene import EdgeMetaGeneGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneGo import EdgeMetaGeneGo
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneInhibitionGene import EdgeMetaGeneInhibitionGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneOverAna import EdgeMetaGeneOverAna
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGenePath import EdgeMetaGenePath
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGenePheno import EdgeMetaGenePheno
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGenePtmodGene import EdgeMetaGenePtmodGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneReactionGene import EdgeMetaGeneReactionGene
from openbiolink.graph_creation.metadata_edge.edge.edgeMetaGeneUnderAna import EdgeMetaGeneUnderAna
from openbiolink.graph_creation.metadata_edge.edge.tnEdgeMetaDisDrug import TnEdgeMetaDisDrug
from openbiolink.graph_creation.metadata_edge.edge.tnEdgeMetaDisPheno import TnEdgeMetaDisPheno
from openbiolink.graph_creation.metadata_edge.edge.tnEdgeMetaGeneAna import TnEdgeMetaGeneAna
from openbiolink.graph_creation.metadata_edge.edge.tnEdgeMetaGeneOverAna import TnEdgeMetaGeneOverAna
from openbiolink.graph_creation.metadata_edge.edge.tnEdgeMetaGeneUnderAna import TnEdgeMetaGeneUnderAna
