DEFAULT_HITS_AT_K = [1, 5, 10, 30, 50]
EVAL_OUTPUT_FOLDER_NAME = 'evaluation'
EVAL_OUTPUT_FILE_NAME = 'evaluation_result.txt'
CORRUPTED_HEADS_FILE_NAME = 'corrupted_heads.csv'
CORRUPTED_TAILS_FILE_NAME = 'corrupted_tails.csv'
CORRUPTED_GROUP_COL_NAME = 'number'

MODEL_DIR = 'model'
MODEL_TRAINED_NAME = 'trained_model.pkl'
MODEL_ENTITY_NAME_MAPPING_NAME = 'entity_name_to_id.json.json'
MODEL_ENTITY_TYPE_MAPPING_NAME = 'entity_type_to_id.json'
MODEL_RELATION_TYPE_MAPPING_NAME = 'relation_type_to_id.json'
