from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapDoAltidProcessor import OntoMapDoAltidProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapDoOmimProcessor import OntoMapDoUmlsProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapDoUmlsProcessor import OntoMapDoUmlsProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapGoAltidProcessor import OntoMapGoAltidProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapHpoAltidProcessor import OntoMapHpoAltidProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapHpoUmlsProcessor import OntoMapHpoUmlsProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapUberonAltidProcessor import OntoMapUberonAltidProcessor
