from openbiolink.graph_creation.file_processor.edge.edgeBgeeExprProcessor import EdgeBgeeExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeBgeeNoExprProcessor import EdgeBgeeNoExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeBgeeOverExprProcessor import EdgeBgeeOverExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeBgeeUnderExprProcessor import EdgeBgeeUnderExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeCdtPathProcessor import EdgeCdtPathProcessor
from openbiolink.graph_creation.file_processor.edge.edgeDisGeNetProcessor import EdgeDisGeNetProcessor
from openbiolink.graph_creation.file_processor.edge.edgeDrugCentralContraIndProcessor import EdgeDrugCentralIndProcessor
from openbiolink.graph_creation.file_processor.edge.edgeDrugCentralIndProcessor import EdgeDrugCentralIndProcessor
from openbiolink.graph_creation.file_processor.edge.edgeGoProcessor import EdgeGoProcessor
#from openbiolink.graph_creation.file_processor.edge.edgeHpaProcessor import EdgeHpaProcessor --> replaced by bgee
from openbiolink.graph_creation.file_processor.edge.edgeHpoDisProcessor import EdgeHpoDisProcessor
from openbiolink.graph_creation.file_processor.edge.edgeHpoGeneProcessor import EdgeHpoGeneProcessor
# from openbiolink.graph_creation.file_processor.edge.edgeSiderIndProcessor import EdgeSiderIndProcessor --> replaced by drugcentral
from openbiolink.graph_creation.file_processor.edge.edgeSiderSeProcessor import EdgeSiderSeProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchActivationProcessor import EdgeStitchActivationProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchBindActProcessor import EdgeStitchBindActProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchBindInhProcessor import EdgeStitchBindInhProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchBindingProcessor import EdgeStitchBindingProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchCatalysisProcessor import EdgeStitchCatalysisProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchExpressionProessor import EdgeStitchExpressionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchInhibitionProcessor import EdgeStitchInhibitionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchPredBindProcessor import EdgeStitchPredBindProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchProcessor import EdgeStitchProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchReactionProcessor import EdgeStitchReactionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringActivationProcessor import EdgeStringActivationProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringBindActProcessor import EdgeStringBindActProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringBindInhProcessor import EdgeStringBindInhProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringBindingProcessor import EdgeStringBindingProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringCatalysisProcessor import EdgeStringCatalysisProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringExpressionProcessor import EdgeStringExpressionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringInhibitionProcessor import EdgeStringInhibitionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringProcessor import EdgeStringProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringPtmodeProcessor import EdgeStringPtmodeProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringReactionProcessor import EdgeStringReactionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeTnHpoDisProcessor import EdgeTnHpoDisProcessor
